/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to get an exact delay
 *    of the execution by using a timer in irq mode.
 *
 *    Observe that at least the interrupt rountines must be compiled in
 *    ARM code, as opposed to THUMB code. The IRQ routines are compiled
 *    in a separate library (irq_code).
 *
 *    Also observe that the startup code framwork must handle the VIC and
 *    dispatch the correct interrupt ISRs. This can be done with Embedded
 *    Artists' startup framework by setting the define: IRQ_HANDLER to 1
 *    (meaning: Load vector directly from VIC, i.e., LDR PC,[PC,#-0xFF0]).
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"
#include "irq_code/irq.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs, void (*pCallback)());
static void callback1(void);
static void callback2(void);


/*****************************************************************************
 * Local variables
 ****************************************************************************/
volatile tU8 timeoutExpired = FALSE;
void (*pDelayCallback)(void) = NULL;


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Callback function that is called when a timeout expires.
 *    Observe that the code is executed in IRQ context so the stack space
 *    usage as well as the execution time should be kept as small as possible.
 *
 ****************************************************************************/
static void
callback1(void)
{
	//do something...
  printf("+");
}


/*****************************************************************************
 *
 * Description:
 *    Callback function that is called when a timeout expires.
 *    Observe that the code is executed in IRQ context so the stack space
 *    usage as well as the execution time should be kept as small as possible.
 *
 ****************************************************************************/
static void
callback2(void)
{
	//do something...
  printf("-");
}


/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #0. An interrupt driven implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *    [in] pCallback - function pointer to timer callback function.
 *                     NULL if no callback function shall be registered.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs, void (*pCallback)())
{
	//register callback and reset expired flag
  pDelayCallback = pCallback;
  timeoutExpired = FALSE;

  //initialize VIC for Timer0 interrupts
  VICIntSelect &= ~0x10;           //Timer0 interrupt is assigned to IRQ (not FIQ)
  VICVectAddr4  = (tU32)timer0ISR; //register ISR address
  VICVectCntl4  = 0x24;            //enable vector interrupt for timer0
  VICIntEnable  = 0x10;            //enable timer0 interrupt
  
  //initialize and start Timer #0
  T0TCR = 0x00000002;                           //disable and reset Timer0
  T0PC  = 0x00000000;                           //no prescale of clock
  T0MR0 = delayInMs *                           //calculate no of timer ticks
         ((CRYSTAL_FREQUENCY * PLL_FACTOR) / (1000 * VPBDIV_FACTOR));
  T0IR  = 0x000000ff;                           //reset all flags before enable IRQs
  T0MCR = 0x00000003;                           //reset counter and generate IRQ on MR0 match
  T0TCR = 0x00000001;                           //start Timer0
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
  //initialize printf()-functionality
  eaInit();

  while(1)
  {
    //wait 375 ms
    delayMs(375, callback1);
    
    //wait for delay... or do something else...
    while (timeoutExpired == FALSE)
      ;
    
    //wait 35 ms
    delayMs(35, callback2);
    
    //wait for delay... or do something else...
    while (timeoutExpired == FALSE)
      ;
  }

  return 0;
}
